<?php
class TA_Chart{
	protected $_graph;
	protected $_chart;
	protected $_rows;
	protected $_columns;
	protected $_display_resources;
	protected $_title;
	protected $_valid_rows;
	protected $_period;			// a=>all, d=>daily, m=>monthly, w=>weekly
	protected $_x_labels = array();
	protected $_total=array();
	
	/*
	 * * Constructor, Order of function call should not be changed
	 */	
	function __construct(){
		
		$this->_graph = new TA_Data_Graph();
		$this->_chart = new open_flash_chart();
		$this->_rows = $this->_graph->get_items();		
		$this->_columns = $this->_graph->get_columns();
		$this->_display_resources = $this->_graph->get_resources();
		$this->set_period();	
			
		$this->valid_resources();
		$this->set_x_labels();
		
		$this->_title = new title($this->get_chart_title($this->_rows));
		$title_style = "font-size:20px;background-color:#ddddaa;padding:20px";
		$this->_title->set_style($title_style);
		
		
		
	}
	/*
	 * Function returns pie chart object, if the number of x-axis labels is one . Otherwise return line chart object
	 */
	function get_chart_type(){
		
		if(count($this->_x_labels)==0)
			$c = new TA_No_Chart();
		else if(count($this->_x_labels)==1 || $this->_period =="a" || !isset($this->_period) || empty($this->_period) )
			$c = new TA_Pie_Chart();
		else if(count($this->_x_labels)>1 )
			$c = new TA_Line_Chart();	
			
		return $c;	
	}
	
	/*
	 * Draw the Pie chart or Line chart based on the argument object
	 * @param TA_Chart $c
	 */
	function draw(TA_Chart $c){
		$c->draw();
	}	
	
	/*
	 * Initializes X Axis labels
	 */
	function set_x_labels(){
		if(!is_array($this->_valid_rows))
			$this->_valid_rows=array();
		foreach($this->_valid_rows as $row){
			$this->_x_labels["$row[0]"] = $row[0];		
		}		
	}	
	
	/*
	 * Initializes the period property
	 */
	function set_period(){
		$this->_period = "";
		if(isset($_GET['period']))
			if(!empty($_GET['period']))
				$this->_period = $_GET['period'];			
	}
	
	/*
	 * Retains the rows, that has to be displayed ( applying the conditions like top3, top5 etc ) 
	 */
	function valid_resources(){
			foreach($this->_rows as $row){
				$this->_total["$row[0]"] += (int) $row[2];		// Total visits of all resources on a specific selected period ( may be daily, monthly or weekly )
				if( ! in_array($row[1], $this->_display_resources) )
					continue;
				if($row[0]=="0")
					continue;			
				$this->_valid_rows[]=$row;
		}		
		//var_dump($this->_valid_resources);
	}
	
	/*
	 * Finding the title of the chart
	 */	
	function get_chart_title(){
		$title = "Views " . $this->get_range() . " " .  $this->get_mode();
		return $title;	
	}

	/*
	 * Finding the range of dates to be displayed in the chart title
	 */
	function get_range(){
		$start="";
		$end="";
		$title = "";
		$cols=0;
		$range = $this->get_limit();
		
				
		$start = $range["start"];
		$end = $range["end"];
				
		if($start==$end)
			$title = " on $start";
		else if( empty($end))
			$title = " on $start";
		else 
			$title = " from " . $start . " to " . $end;
		
		return $title;
		
	}
	
	/*
	 * Finding the start and end of range to be displayed in title chart
	 */
	function get_limit(){
		$i=0;
		$start="";
		$end="";
		
		$min=9999999999;
		$max=0;
		
		if($this->_period=="a" || empty($this->_period)){
			foreach($this->_valid_rows as $row){
				if($row[4]<$min){
					$min = $row[4];
					$start = $row[5];
				}
				if($row[6]>$max){
					$max = $row[6];
					$end = $row[7];
				}
			}			
			
			
		}else {
			foreach($this->_x_labels as $x){
					if($i==0)			
						$start = $x;
					$i++;
			}
			$end = $x;
		}
		return array("start"=>$start,"end"=>$end);
	}
	
	/*
	 * Calculating the color of resource to be displayed in the chart
	 */	
	function get_color($num){
		$hex = hexdec($num*1000000);
		$hex = "#".$hex;
		$hex = str_pad($hex,7,"0",STR_PAD_RIGHT);
		return $hex;	
	}
	
	/*
	 * Finding the mode to be displayed in the Chart title
	 */
	function get_mode(){
		$title = "";
		switch($this->_period){
			case "a":
				$title = "";
				break;
			case "m":
				$title = "(Monthly)";
				break;
			case "w":
				$title = "(Weekly)";
				break;
			case "d":
				$title = "(Daily)";
				break;
		}		
		return $title;
	}	
}