<?php

class TA_Data_Graph extends TA_Data {

	function __construct() {
		$this->init_period();	
		$this->init_dates();	
		$this->set_filters();
		$this->set_columns();		
		$this->set_items();		
	}

	/*
	 * Function to initialize the entered date related fields
	 */	
	function init_dates(){
		if(isset($_GET['daterangepicker'])){
			$this->_entered_date = $_GET['daterangepicker'];
		}
		
		if(isset($_GET['daterangepicker_start'])){
			$this->_start_date = $_GET['daterangepicker_start'];
		}
		
		if(isset($_GET['daterangepicker_end'])){
			$this->_end_date = $_GET['daterangepicker_end'];
		}		
	}
		
	/*
	 * Function to initialize the period field
	 */	
	function init_period(){
		if(isset($_GET['period']))
			$this->_period = $_GET['period'];
	}
	
	
	/*
	 * Function to define the column headings [ Query Select Fields ] 
	 * 
	 */	
	function set_columns(){	
		$period = array();
		global $wpdb;
		
		$this->_cols = array (
			array("resource","Resource","post_title as resource"),		// Column 1 : Post Title
			array("cnt","Count","ifnull(count(resource),0) as cnt"),	// Column 2 : View Count
			array("id","PostID","{$wpdb->prefix}posts.id as id")		// Column 3 : Post Id
		);
		
		if($this->is_daily($this->_period)){
			$period=array("period","Day","ifnull(date_format(vtime,'%d-%M-%Y'),0) as period");
			array_unshift($this->_cols,$period);	// Column 0 : Period [ eg : 04-Oct-1978 ] 
		}else if($this->is_all($this->_period)){
			$period=array();
				
			$period=array("period","Day","ifnull(date_format(vtime,'%d-%M-%Y'),0) as period");
			array_unshift($this->_cols,$period);		// Column 0	: Period [ eg : 04-Oct-1978 ]
				
			$period=array("period_start_unix","period_start_unix","   ifnull(unix_timestamp(min(date(vtime))),0) as period_start_unix");
			array_push($this->_cols,$period);	// Column 4 : Start date of Views in Unix Time stamp
				
			$period=array("period_start","period_start"," ifnull(date_format(min(date(vtime)),'%d-%M-%Y'),0) as period_start");
			array_push($this->_cols,$period);	// Column 5 : Start date of Views
				
			$period=array("period_end_unix","period_end_unix"," ifnull(unix_timestamp(max(date(vtime))),0) as period_end_unix");
			array_push($this->_cols,$period);	// Column 6 : End date of views in Unix Time stamp			
				
			$period=array("period_end","period_end"," ifnull(date_format(max(date(vtime)),'%d-%M-%Y'),0) as period_end");
			array_push($this->_cols,$period);	// Column 7 : End date of Views
								
		}else if($this->is_weekly($this->_period)){
			$period=array("period","Week","ifnull(date_format(vtime,'%u'),0) as period");
			array_unshift($this->_cols,$period);	// Column 0  : Period [ Week ]

			$period=array("period_year","period_year","ifnull(date_format(vtime,'%Y'),0) as period_year");
			array_push($this->_cols,$period);	// Column 4  : Period [ Year ]
			
			
		}else if($this->is_monthly($this->_period)){
			$period=array("period","Month","ifnull(date_format(vtime,'%M %Y'),0) as period");
			array_unshift($this->_cols,$period);	// Column 0 : Period [ Mon Year ] 
		}
	}

	
	/*
	 * Function to define the filters
	 */	
	function set_filters(){	// 
		$this->_filters = array(
			array("id"=>"daterangepicker","type"=>"text", "where_daterangepicker"=>"type:date_range;table_field:date(vtime);start_filter:daterangepicker_start;end_filter:daterangepicker_end" ),		// DateRange Picker
			array("id"=>"daterangepicker_start","type"=>"hidden"), 			// Hidden, DateRange Start Field
			array("id"=>"daterangepicker_end","type"=>"hidden"),				// Hidden, DateRange End Field
			array("id"=>"period","type"=>"select","options"=>array(
																array("value"=>"a","text"=>"All"),
																array("value"=>"d","text"=>"Daily","group_by_d"=>"date_format(vtime,'%d-%m-%Y')"),
																array("value"=>"w","text"=>"Weekly","group_by_w"=>"date_format(vtime,'%u %Y')"),
																array("value"=>"m","text"=>"Monthly","group_by_m"=>"date_format(vtime,'%m %Y')"),
														) 
												),		// Period
			array("id"=>"filter","type"=>"submit", "value"=>"Filter")	
		);
		
	}

	/*
	 * Draws the Chart Page
	 */
	function display() {
		$this->show_filters();
?>
	<div id="my_chart"></div>
<?php

	}

	/*
	 *	Returns the all the items matching the request 
	 */
	function get_items() {
		return $this->items;
	}

	/*
	 * Returns the resources, matching the settings
	 * 
	 */	
	function get_resources(){
		global $current_user;
		$res=array();
		$where = $this->generate_where();
		$where .= " and post_status='publish' and post_type in ( 'post' , 'page' ) " ;
		$group_by = "group by resource ";
		$order_by = "";
		$ta_titles =  get_option("ta_titles");
		$title = $ta_titles["$current_user->ID"];
		$ta_titles_id = get_option("ta_titles_id");		// Custom Titles
		if(isset($ta_titles_id[$current_user->ID]) && is_array($ta_titles_id[$current_user->ID])){
			$id_array = $ta_titles_id[$current_user->ID];
			$id = implode(",",$id_array);
			$id = "(".$id.")";
		}else{
			$id = "(0)";
		}
		
		if(!isset($title) || empty($title))
			$title = TrafficAnalyzer::DEFAULT_NO_OF_TITLES;
		
		$limit = "";
		
		switch($title){
			case "1":
				$order_by = " order by cnt desc ";
				$limit= " limit 1";
				break;
			case "3":
				$order_by = " order by cnt desc ";
				$limit= " limit 3";
				break;	
			case "5":
				$order_by = " order by cnt desc ";
				$limit= " limit 5";
				break;
			case "7":
				$order_by = " order by cnt desc ";
				$limit= " limit 7";
				break;
			case "10":
				$order_by = " order by cnt desc ";
				$limit= " limit 10";
				break;			
			case "a":
				$order_by = " order by cnt desc ";
				$limit= " ";
				break;
			case "c":
				$order_by = " order by cnt desc ";
				$limit= " ";
				$where .= " and wp_posts.id in $id " ;
				break;
				
				
		}
		
		global $wpdb;
		$sql = " select post_title,ifnull(count(resource),0) as cnt,wp_posts.id as id from {$wpdb->prefix}tanalyzer_visits right outer join wp_posts on ( resource=wp_posts.id) $where $group_by $order_by $limit ";
		$result = $wpdb->get_results($sql);

		if(!is_array($result))
			$result = array();
		foreach ($result as $r){
			$res[$r->id] = $r->post_title;				
		}
		
		return $res;	
	}	
}
?>
