<?php

class TA_Data  {
	
	protected $_entered_date="";
	protected $_start_date="";
	protected $_end_date="";
	protected $_period="";	
	
	protected $items=array();
	
	protected $_cols="";
		
	protected $_filters=array();
	
	private $_group_by = "";
	
	private $_where = array();

	
	/*
	 * Constructor to this class
	 */	
	function __construct() {
	}
	
	/*
	 * Function that returns the start date
	 */
	function get_start_date(){
		return $this->_start_date;
	}
	
	
	/*
	 * Function that returns the end date
	 */
	function get_end_date(){
		return $this->_end_date;
	}
	
	

	/*
	 * Function that returns the array containing all the columns
	 * @return array
	 */	
	function get_columns() {
		$posts_columns = array();
		if(!is_array($this->_cols))
			$this->_cols=array();
		foreach($this->_cols as $col){
			$posts_columns[$col[0]]=$col[1];
		}	
		return $posts_columns;
	}	
	
	/*
	 * Function that converts the filters defined in the array $this->_filters in to corresponding html
	 * @return string, 
	 */
	function show_filters() {
		
		?>
					
		<div>
		<?php

		if(!is_array($this->_filters))
			$this->_filters = array();
		
		foreach($this->_filters as $filter){
			switch($filter["type"]){
				case "text":
					?>
					<input type="text" readonly id="<?php echo $filter["id"]; ?>" name="<?php echo $filter["id"];?>" value="<?php echo $_GET[$filter["id"]]; ?>" />					
					<?php 
					break;
				case "hidden":
					?>
					<input type="hidden" id="<?php echo $filter["id"]; ?>" name="<?php echo $filter["id"];?>" value="<?php echo $_GET[$filter["id"]]; ?>" />
					<?php 
					break;
				case "select" :
					$filter_id = $filter["id"];
					
					if(isset($_GET["$filter_id"])){
						$selected[$_GET["$filter_id"]]="selected";		
					}
					
					
					?>
					<select id="<?php echo $filter["id"]; ?>" name="<?php echo $filter["id"];?>" >
					<?php 
						foreach($filter['options'] as $option){
							?>
								<option  value="<?php echo $option['value']; ?>" <?php $value = $option['value']; echo $selected["$value"]; ?>   ><?php echo $option['text']; ?></option>							
							<?php 	
						}	
						if( ! @empty($filter['function_name'])) {
							$options = $this->$filter['function_name']();		
							foreach($options as $option){
							?>
								<option <?php $value = $option; echo $selected["$value"]; ?> ><?php echo $option;?></option>							
							<?php 
							}
						}
					?>					
					</select>
					<?php 
					break;
				case "submit" :
					?>
					<input type="submit" class="button-secondary" value="<?php echo $filter['value']; ?>" />
					<?php
			}
		}
		?>
		</div>
		<?php 
		
	}
	

	/*
	 * Function to determine the first day for the week or month corresponding to the argument date, assuming monday is the first day of the week
	 * @param string $dt , date in format "yyyy/mm/dd" 
	 * @param string $period, m for first day of month , w for first day of week
	 * @return string ,  first day of the week for the argument date
	 */	
	function get_first_day($dt,$period){
		if($this->is_monthly($period)){	// For first day of month
			$dt = substr($dt,0,4)."/".substr($dt,5,2)."/01";
		}else if($this->is_weekly($period)){	// For first day of week		
			$ts = mktime(0,0,0,substr($dt,5,2),substr($dt,8,2),substr($dt,0,4));
			$week = date("W",$ts);
			$year = date("Y",$ts);			
			$wk_ts  = strtotime('+' . $week . ' weeks', strtotime($year . '0101'));
	    	$mon_ts = strtotime('-' . date('w', $wk_ts) + 1  . ' days', $wk_ts);	// monday timestamp	    	
	    	$dt = date('Y-m-d', $mon_ts);    	
		}
		return $dt;
	}
	
	/*
	 * Function to determine the last day for the week corresponding to the argument date, assuming the sunday is the last day of the week
	 * @param string $dt , date in format "yyyy/mm/dd" 
	 * @return string ,  last day of the week for the argument date
	 */	
	function get_last_day($dt,$period){
		if($this->is_weekly($period)){
			$ts = mktime(0,0,0,substr($dt,5,2),substr($dt,8,2),substr($dt,0,4));
			$week = date("W",$ts);
			$year = date("Y",$ts);			
			$wk_ts  = strtotime('+' . $week . ' weeks', strtotime($year . '0101'));
			$sun_ts =  strtotime('+' . (7 - date('w', $wk_ts) ) . ' days', $wk_ts);		// sunday timestamp ( Sunday is the last day of the week ) 
	    	$dt = date('Y-m-d',  $sun_ts );
		}else if($this->is_monthly($period)) {
			$dt_ts = strtotime($dt);
			$last_day = date("t",$dt_ts);
			$dt = substr($dt,0,4)."/".substr($dt,5,2)."/".$last_day;			
		}
		return $dt;
	}
	
	/*
	 * callback function used in generate_where()
	 */	
	function get_where($value,$key){
					
		if($this->has_where($key)){			
			$fields = explode(";",$value);				
			$data = array();
			foreach($fields  as $field){
				$pos = strpos($field, ":");
				$data[substr($field, 0, $pos) ] = substr($field, $pos+1) ;					
			}

			$table_field = $data['table_field'];
			$start_filter = $data['start_filter'];
			$end_filter = $data['end_filter'];			
				
			if($this->is_date_range()){
				$this->_where[$key] = " date($table_field)  between  '".  $this->get_first_day($this->_start_date,$this->_period) ."' and '" . $this->get_last_day($this->_end_date,$this->_period) ."'";						
			}else if($this->is_single_date()) {
				$this->_where[$key] = " date($table_field)  between  '".  $this->get_first_day($this->_start_date,$this->_period) ."' and '" . $this->get_last_day($this->_start_date,$this->_period) . "'" ;
			}
		}
	}
	
	
	/*
	 * Function to generate where clause to be used in sql select statement
	 * @return string, where clause
	 */
	function generate_where(){		
		$this->_where = array();
		$where = " where  1 = 1 ";

		if(!is_array($this->_filters))
			$this->_filters=array();
		foreach($this->_filters as $filter){		
				array_walk_recursive($filter, array($this,'get_where'));						
		}		
		foreach($this->_where as $w){
			$where .= " and ". $w;	
		}		
		return $where;		
	}

	
	/*
	 * callback function used in generate_group_by()
	 */	
	function get_group_by($value,$key,$data){ 
		if($key===$data){
			$this->_group_by = $value;	
		}			
	}
	
	/*
	 * Function to generate group by clause to be used in sql select statement
	 * @return string, group by clause
	 */	
	function generate_group_by(){
		$group_by = "group by resource";	// Always be grouped with resource
		if(!empty($this->_period)){					
			$this->_group_by = "";
			array_walk_recursive($this->_filters, array($this,"get_group_by"),"group_by_".$this->_period);
			if($this->_group_by!="")
				$group_by .= ", ". $this->_group_by;
			else 	
				$group_by .= "";						
		}else {
			$group_by .= "";		
		}
		return $group_by;		
	}
	
	
	/*
	 * Function to generate columns to be used in sql select statement
	 * @return string, columns separated by comma ( columns should be already defined in the array $this->_cols ) 
	 */
	function generate_select_fields(){
		foreach ($this->_cols as $field){
			$select[] = $field[2];
			$id[] = $field[0];
		}
		$select_fields = implode(",",$select);
		return $select_fields;		
	}
	
	function generate_order_by(){
		if($this->is_monthly($this->_period))
			$order_by = " order by date_format(vtime,'%m %Y') ";
		else if($this->is_daily($this->_period))
			$order_by = "order by date(vtime)";
		else if($this->is_weekly($this->_period))
			$order_by = "order by period ";
		else 
			$order_by = "order by cnt desc";
		return $order_by;		
	}	
	
	
	
	/*
	 * Function to generate SQL statement
	 * @return string, SQL select statement
	 */
	
	function generate_sql(){
		global $user_agent;
		$cols = $this->get_columns();
		$where = "";

		$order_by = $this->generate_order_by();
		$group_by = "";
		$select_fields = "";				
		$where .= $this->generate_where();		
		$group_by = $this->generate_group_by();
		$select_fields = $this->generate_select_fields();
		
		$where .= " and post_status='publish' and post_type in ( 'post' , 'page' )   " ;
		
		foreach($user_agent as $agent){
			$where .= " and user_agent not like '%$agent%' ";
		}		
			
		$where .= " and user_agent<>''";
		
		global $wpdb;			
		
		$sql = " select $select_fields from {$wpdb->prefix}tanalyzer_visits right outer join {$wpdb->prefix}posts on ( resource={$wpdb->prefix}posts.id) $where $group_by  $order_by  ";
		return $sql;
	}
	
	function set_items() {		
		global $wpdb;
		$offset = 0;

		if(!is_array($this->_cols))
			$this->_cols = array();
		
		foreach ($this->_cols as $field){
			$id[] = $field[0];
		}
		
		$sql = $this->generate_sql();	
				
		$result = $wpdb->get_results($sql);
		
		if(!is_array($result))
			$result = array();
		
		$no=$offset+1;
		foreach($result as $row){
			$r=array();
			foreach($id as $col){
				if($col=="no")
					$r[]=$no;
				else				
					$r[] = $row->$col;				
					
			}
			$this->items[] = $r;
			$no++;
		}
	}
	
	/*
	 * Checks whether requested is daily or not
	 * @param string $period
	 * @return bool
	 */	
	function is_daily($period){
		if($period=="d")
			return true;
		else 
			return false;
	}
	
	
	/*
	 * Checks whether requested is weekly or not
	 * @param string $period
	 * @return bool
	 */
	function is_weekly($period){
		if($period=="w")
			return true;
		else 
			return false;
	}

	/*
	 * Checks whether requested is monthly or not
	 * @param string $period
	 * @return bool
	 */
	function is_monthly($period){
		if($period=="m")
			return true;
		else 
			return false;
	}
	
	/*
	 * Checks whether requested is all or not
	 * @param string $period
	 * @return bool
	 */
	function is_all($period){
		if($period=="a" || empty($period) )
			return true;
		else 
			return false;
	}

	
	/*
	 * Checks whether the $key of the current filter has a where condition defined
	 */	
	function has_where($key){
		if(substr($key,0,6)=="where_")
			return true;
		return false;
		
	}
	
	
	/*
	 * Checks whether start_date and end_date are set
	 * 
	 */	
	function is_date_range(){
		if(!(empty($this->_end_date)))
			return true;
		return false;
	}
	
	/*
	 * Checks whether start_date is set
	 * 
	 */	
	function is_single_date(){
		if(!(empty($this->_start_date)))
			return true;
		return false;
	}
		
}
?>
