<?php
class TA_Line_Chart extends TA_Chart {
	private $_y_values = array();
	private $_colors = array();
	private $_resource_labels = array();
	private $_years = array();
	
	/*
	 * Constructor, Order of function call should not be changed
	 */
	function __construct(){
		parent::__construct();			
		$this->set_x_labels();			
		$this->set_y_values();
		$this->set_colors();
		$this->set_resource_labels();
		if($this->_period == 'w')
			$this->set_years();	// Should be called only for periodicity is weekly		
	}
	
	
	/*
	 * Determining the Y-Axis Values 
	 * _y_values[period][article_title]
	 */
	function set_y_values(){
		foreach($this->_valid_rows as $row){
			$this->_y_values["$row[0]"]["$row[1]"]=$row[2];		
		}		

	}
	
	/*
	 * Determining the Year for Periodicity is Weekly  
	 * _year[period][article_title]
	 */
	function set_years(){
		foreach($this->_valid_rows as $row){
			$this->_years["$row[0]"]["$row[1]"]=$row[4];		//$row[4] will be available only for weekly		
		}
	}	
	
	/*
	 * Setting the colour of resource to an array
	 */
	function set_colors(){
		foreach($this->_valid_rows as $row){
			$this->_colors["$row[1]"]=$this->get_color($row[3]);		
		}		
	}
	
	/*
	 * Determining the Resource Labels
	 */
	function set_resource_labels(){
		foreach($this->_valid_rows as $row){
			$this->_resource_labels["$row[1]"]=$row[1];		
		}		
	}
	
	/*
	 * Drawing the Line chart
	 */
	function draw() {
		$min_range=0;
		$max_range=0;
		$no_step=10;
		$step=0;
		$data = array();				
		
		/*
		 *  Creating Json data for Passing to javascript function of onclick event
		 */
		$grid_data = array();
		$sdate=$this->_graph->get_start_date();
		$edate=$this->_graph->get_end_date();
		
		if(isset($sdate) && !empty($sdate))
			$grid_data['sdate']=$sdate;
		else 
			$grid_data['sdate']="";
			
		if(isset($edate) && !empty($edate))
			$grid_data['edate']=$edate;
		else 
			$grid_data['edate']="";

		$grid_data['period'] = $this->_period;		// d or w or m
		
		//$grid_data['period_title'] = $this->get_chart_title();
		
		
		
		foreach($this->_resource_labels as $r){		// Article Titles
			
			$data = array();
			if(strlen($r)>17)
				$dot_title = "(".substr($r,0,17)."...".")";
			else 
				$dot_title = "(".$r.")";
										
			foreach($this->_x_labels as $x) {		// Date, week or month
					
				if(isset($this->_y_values["$x"]["$r"])){

					$dot = new solid_dot((int)$this->_y_values["$x"]["$r"]);
					$dot->tooltip($this->_y_values["$x"]["$r"]."/". $this->_total[$x] .$dot_title);

					/* Grid Data */
					$keys = array_keys($this->_display_resources,$r,true);
					$grid_data["key"] = $keys[0];	// post_id
					$grid_data["article"] = $r;		// post_title
					
					if($this->_period=='d' ){
                    	$grid_data['sdate'] = date('Y/m/d',strtotime($x));
                        $grid_data['edate'] = date('Y/m/d',strtotime($x));
                        $grid_data['period_title'] = "Views on $x";
                    }else if($this->_period=='w') {
                    	$week = $x;
                        $year = $this->_years[$x][$r];                                          
                        $grid_data['sdate'] = ta_week_start_date($week, $year);                                         
                        $grid_data['edate'] = date('Y/m/d', strtotime('+6 days', strtotime($grid_data['sdate'])));
                        $grid_data['period_title'] = "Views on week $x (".date('d-M-Y',strtotime($grid_data['sdate']))." - ". date('d-M-Y',strtotime($grid_data['edate'])). ")";                                                                                                
                    }else if($this->_period == 'm') {
                    	$grid_data['sdate'] = date('Y/m/d',strtotime("01"." ".$x)) ;// Space is the separator                                           
                    	$grid_data['edate'] = date('Y/m/d',strtotime(date('t', strtotime($grid_data['sdate']." ".$x)))) ;
						$grid_data['period_title'] = "Views on $x";
                    }				
										
					$data_json = json_encode($grid_data);
						
					$dot->on_click("show_dialog($data_json)");
					
					array_pop($grid_data);
					array_pop($grid_data);
					$data_json="";
					/* Grid Data */									
					
					$data[]= $dot;
					
					//setting max_range
					if($max_range<(int)$this->_y_values["$x"]["$r"])
						$max_range = (int) $this->_y_values["$x"]["$r"];	
				}else{
					$dot = new solid_dot(0);
					$dot->tooltip("0"."/". $this->_total[$x]."$dot_title");
					$data[]=$dot;
				}		
			}			
			
			$line = new line();
			$line->set_values($data);
			
			$line->set_text($r);
			$line->set_colour($this->_colors[$r]);
			$this->_chart->add_element($line);					
		}
		
		$step = ( $max_range  -  $min_range ) / $no_step;
		
		foreach($this->_x_labels as $x){
			$lbl[] = $x;
		}
		
		$x_axis_labels = new x_axis_labels();
		$x_axis_labels->set_labels($lbl);
		$x_axis_labels->rotate(320);
		$x_axis = new x_axis();
		$x_axis->set_labels($x_axis_labels);
			
		$y_axis = new y_axis();
		$y_axis->set_range($min_range, $max_range,$step);

		$this->_chart->set_x_axis($x_axis);
		$this->_chart->set_y_axis($y_axis);
		$this->_chart->set_title($this->_title);
		echo $this->_chart->toString();
	}
}